@@ -16,7 +16,6 @@
 #undef HAVE_CHPASS
 #undef HAVE_CLEARENV
 #undef HAVE_CRYPT
-#undef HAVE_CTRUN
 #undef HAVE_DEFOPEN
 #undef HAVE_FBCONSOLE
 #undef HAVE_GETTEXT
@@ -36,15 +35,12 @@
 #undef HAVE_SOLARIS_XINERAMA
 #undef HAVE_STPCPY
 #undef HAVE_SYS_SOCKIO_H
-#undef HAVE_SYS_VT_H
 #undef HAVE_TCPWRAPPERS
 #undef HAVE_UNSETENV
+#undef HAVE_UT_SYSLEN
 #undef HAVE_XINERAMA
 #undef HAVE_XFREE_XINERAMA
 #undef HAVE_XINPUT
-#undef UTMP_LINE_ATTACHED
-#undef UTMP_LINE_REMOTE
-#undef UTMP_PSEUDO_DEVICE
 #undef X_SERVER
 #undef X_SERVER_PATH
 #undef X_XNEST_CMD
@@ -54,4 +50,3 @@
 #undef X_PATH
 #undef XSESSION_SHELL
 #undef XEVIE_OPTION
-#undef HAVE_UT_SYSLEN
