@@ -52,11 +52,11 @@ enum {
         PROP_UID,
         PROP_HOME_DIR,
         PROP_SHELL,
-        PROP_IS_LOGGED_IN,
 };
 
 enum {
         ICON_CHANGED,
+        SESSIONS_CHANGED,
         LAST_SIGNAL
 };
 
@@ -70,7 +70,7 @@ struct _GdmUser {
         char           *real_name;
         char           *home_dir;
         char           *shell;
-        gboolean        is_logged_in;
+        GSList         *sessions;
 };
 
 typedef struct _GdmUserClass
@@ -78,6 +78,7 @@ typedef struct _GdmUserClass
         GObjectClass parent_class;
 
         void (* icon_changed)     (GdmUser *user);
+        void (* sessions_changed) (GdmUser *user);
 } GdmUserClass;
 
 static void gdm_user_finalize     (GObject      *object);
@@ -86,6 +87,64 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GdmUser, gdm_user, G_TYPE_OBJECT);
 
+static int
+session_compare (const char *a,
+                 const char *b)
+{
+        if (a == NULL) {
+                return 1;
+        } else if (b == NULL) {
+                return -1;
+        }
+
+        return strcmp (a, b);
+}
+
+void
+_gdm_user_add_session (GdmUser    *user,
+                       const char *ssid)
+{
+        GSList *li;
+
+        g_return_if_fail (GDM_IS_USER (user));
+        g_return_if_fail (ssid != NULL);
+
+        li = g_slist_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
+        if (li == NULL) {
+                g_debug ("GdmUser: adding session %s", ssid);
+                user->sessions = g_slist_prepend (user->sessions, g_strdup (ssid));
+                g_signal_emit (user, signals[SESSIONS_CHANGED], 0);
+        } else {
+                g_debug ("GdmUser: session already present: %s", ssid);
+        }
+}
+
+void
+_gdm_user_remove_session (GdmUser    *user,
+                          const char *ssid)
+{
+        GSList *li;
+
+        g_return_if_fail (GDM_IS_USER (user));
+        g_return_if_fail (ssid != NULL);
+
+        li = g_slist_find_custom (user->sessions, ssid, (GCompareFunc)session_compare);
+        if (li != NULL) {
+                g_debug ("GdmUser: removing session %s", ssid);
+                g_free (li->data);
+                user->sessions = g_slist_delete_link (user->sessions, li);
+                g_signal_emit (user, signals[SESSIONS_CHANGED], 0);
+        } else {
+                g_debug ("GdmUser: session not found: %s", ssid);
+        }
+}
+
+guint
+gdm_user_get_num_sessions (GdmUser    *user)
+{
+        return g_slist_length (user->sessions);
+}
+
 static void
 gdm_user_set_property (GObject      *object,
                        guint         param_id,
@@ -137,9 +196,6 @@ gdm_user_get_property (GObject    *object,
         case PROP_SHELL:
                 g_value_set_string (value, user->shell);
                 break;
-        case PROP_IS_LOGGED_IN:
-                g_value_set_boolean (value, user->is_logged_in);
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
                 break;
@@ -211,6 +267,14 @@ gdm_user_class_init (GdmUserClass *class)
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
+        signals [SESSIONS_CHANGED] =
+                g_signal_new ("sessions-changed",
+                              G_TYPE_FROM_CLASS (class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmUserClass, sessions_changed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
 }
 
 static void
@@ -219,7 +283,7 @@ gdm_user_init (GdmUser *user)
         user->manager = NULL;
         user->user_name = NULL;
         user->real_name = NULL;
-        user->is_logged_in = FALSE;
+        user->sessions = NULL;
 }
 
 static void
@@ -444,14 +508,6 @@ gdm_user_get_shell (GdmUser *user)
         return user->shell;
 }
 
-gboolean
-gdm_user_is_logged_in (GdmUser *user)
-{
-        g_return_val_if_fail (GDM_IS_USER (user), FALSE);
-
-        return user->is_logged_in;
-}
-
 gint
 gdm_user_collate (GdmUser *user1,
                   GdmUser *user2)
